/* GtkTetcolor
 * Copyright (C) 1999 Andrey V. Panov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef USE_GNOME
# include <gnome.h>
#else
# include <gtk/gtk.h>
#endif
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "main.h"
#include "score.h"
#include "game.h"
#include "preferences.h"
#include "pixmaps.h"

gint cell_width, left_margin, initial_level, use_graykeys;
#ifdef USE_GNOME
gboolean sound_on;
#endif
gint nav_keys[4], alt_nav_keys[4];
GtkStyle *label_style;
GdkFont *font;
gchar *font_name;
gchar *label_name[MAX_LABEL];	/* Label identifiers */

gboolean preferences_changed;
gboolean preferences_dialog_shown = FALSE, about_dialog_shown = FALSE;
#ifndef USE_GNOME
gboolean scores_dialog_shown = FALSE;
#endif
gboolean help_dialog_shown = FALSE;


gboolean
on_main_window_delete_event (GtkWidget * widget,
			     GdkEvent * event, gpointer user_data)
{
  before_exit ();
  gtk_exit (0);
  return FALSE;
}


#ifdef USE_GNOME
void
on_New_activate (GtkWidget * menuitem, gpointer user_data)
{
  init_game (menuitem);
}


void
on_Scores_activate (GtkWidget * menuitem, gpointer user_data)
{
  gnome_scores_display (_("Gtktetcolor"), "gtktetcolor", NULL, 0);
}


void
on_new_button_clicked (GtkWidget * button, gpointer user_data)
{
  init_game (button);
}


void
on_scores_button_clicked (GtkWidget * button, gpointer user_data)
{
  gnome_scores_display (_("Gtktetcolor"), "gtktetcolor", NULL, 0);
}
#else  

void
on_New_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  init_game (GTK_WIDGET (menuitem));
}


void
on_Scores_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *dialog;

  if (scores_dialog_shown)
    return;
  scores_dialog_shown = TRUE;
  dialog = create_scores_dialog ();
  gtk_widget_show_all (dialog);
}


void
on_new_button_clicked (GtkButton * button, gpointer user_data)
{
  init_game (GTK_WIDGET (button));
}


void
on_scores_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *dialog;

  if (scores_dialog_shown)
    return;
  scores_dialog_shown = TRUE;
  dialog = create_scores_dialog ();
  gtk_widget_show_all (dialog);
}
#endif  


void
#ifdef USE_GNOME
on_Quit_activate (GtkWidget * menuitem, gpointer user_data)
#else
on_Quit_activate (GtkMenuItem * menuitem, gpointer user_data)
#endif
{
  before_exit ();
  gtk_exit (0);
}


void
#ifdef USE_GNOME
on_About_activate (GtkWidget * menuitem, gpointer user_data)
#else
on_About_activate (GtkMenuItem * menuitem, gpointer user_data)
#endif
{
  GtkWidget *about;

#ifndef USE_GNOME
  if (about_dialog_shown)
    return;
  about_dialog_shown = TRUE;
#endif
  about = create_about_dialog ();
  gtk_widget_show_all (about);
}


void
#ifdef USE_GNOME
on_pause_button_clicked (GtkWidget * button, gpointer user_data)
#else
on_pause_button_clicked (GtkButton * button, gpointer user_data)
#endif
{
  GtkWidget *dialog;

  if (continue_game)
    {
      dialog = create_pause_dialog ();
      if (timeout)
	gtk_timeout_remove (timeout);
      if (level_timeout)
	gtk_timeout_remove (level_timeout);
      continue_game = 0;
      gtk_widget_show_all (dialog);
    }
}

void
#ifdef USE_GNOME
on_exit_button_clicked (GtkWidget * button, gpointer user_data)
#else
on_exit_button_clicked (GtkButton * button, gpointer user_data)
#endif
{
  before_exit ();
  gtk_exit (0);
}

#ifndef USE_GNOME
void
on_ok_about_button_clicked (GtkButton * button, gpointer user_data)
{
  about_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}
#endif

gboolean
on_main_window_key_press_event (GtkWidget * widget,
				GdkEventKey * event, gpointer user_data)
{
  gint key = event->keyval;

  if (!timeout || !continue_game)
    return TRUE;
  if (key == nav_keys[0] || key == alt_nav_keys[0])
    {
      if (shift_block (1))
	redraw_cells ();
    }
  else if (key == nav_keys[1] || key == alt_nav_keys[1])
    {
      if (shift_block (-1))
	redraw_cells ();
    }
  else if (key == nav_keys[2] || key == alt_nav_keys[2])
    {
      if (block.type)
	if (rotate_block (rotate_cell_left))
	  redraw_cells ();
    }
  else if (key == nav_keys[3] || key == alt_nav_keys[3])
    {
      if (block.type)
	if (rotate_block (rotate_cell_right))
	  redraw_cells ();
    }
  else if (key == GDK_space)
    {
      if (timeout)
	{
	  gtk_timeout_remove (timeout);
	  interval = INI_INTERVAL / 100;
	  timeout = gtk_timeout_add (interval, timeout_callback,
				     main_window);
	}
      else
	return TRUE;
    }
  return FALSE;
}

#ifndef USE_GNOME
void
on_scores_dialog_button_clicked (GtkButton * button, gpointer user_data)
{
  scores_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

void
scores_dialog_close (GtkWidget *dialog, gpointer user_data)
{
  scores_dialog_shown = FALSE;
}

void
about_dialog_destroy (GtkWidget *dialog, gpointer user_data)
{
  about_dialog_shown = FALSE;
}

void
on_name_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *widget;
  gchar *str;

  widget = (GtkWidget *) gtk_object_get_data (GTK_OBJECT
					      (gtk_widget_get_toplevel
					       (GTK_WIDGET (button))),
					      "name_entry");
  str = gtk_entry_get_text (GTK_ENTRY (widget));
  if (strlen (str) == 0)
    {
      str = g_get_real_name ();
      if (strlen (str) == 0)
	str = g_get_user_name ();
    }
  strcpy (new_name, str);
/*  if (str)
    g_free (str);
 */    
  strcpy (name[name_i], new_name);
  saved_score[name_i] = score;
  choose_score_file ();
  write_score ();
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

void
on_pause_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *pause_dialog;

  continue_game = 1;
  pause_dialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
  interval = INI_INTERVAL - INI_INTERVAL * (level - 1) / 10;
  timeout =
    gtk_timeout_add (interval, timeout_callback, main_window);
  level_timeout =
    gtk_timeout_add (LEVEL_INT, change_level, main_window);
  gtk_widget_destroy (pause_dialog);
}
#endif

void
pause_dialog_destroy (GtkWidget *widget, gpointer user_data)
{
  if (!continue_game){
    continue_game = 1;
    interval = INI_INTERVAL - INI_INTERVAL * (level - 1) / 10;
    timeout =
      gtk_timeout_add (interval, timeout_callback, main_window);
    level_timeout =
      gtk_timeout_add (LEVEL_INT, change_level, main_window);
  }
}


void
#ifdef USE_GNOME
on_pause_activate (GtkWidget *menuitem, gpointer user_data)
#else
on_pause_activate (GtkMenuItem *menuitem, gpointer user_data)
#endif
{
  GtkWidget *dialog;

  if (continue_game)
    {
      dialog = create_pause_dialog ();
      if (timeout)
	gtk_timeout_remove (timeout);
      if (level_timeout)
	gtk_timeout_remove (level_timeout);
      continue_game = 0;
      gtk_widget_show_all (dialog);
    }
}


void
#ifdef USE_GNOME
help_on_keys_activate (GtkWidget *menuitem, gpointer user_data)
#else
help_on_keys_activate (GtkMenuItem *menuitem, gpointer user_data)
#endif
{
  GtkWidget *dialog;

  if (help_dialog_shown)
    return;
  help_dialog_shown = TRUE;
  dialog = create_help_dialog ();
  gtk_widget_show_all (dialog);
}

#ifndef USE_GNOME
void
on_help_button_clicked (GtkButton * button, gpointer user_data)
{
  help_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}
#endif

void
help_dialog_destroy (GtkWidget *widget, gpointer user_data)
{
  help_dialog_shown = FALSE;
}

gboolean
on_main_window_expose_event (GtkWidget * widget,
			     GdkEventExpose * event, gpointer user_data)
{
  redraw_all_cells ();
  return FALSE;
}


void
#ifdef USE_GNOME
on_preferences_activate (GtkWidget * menuitem, gpointer user_data)
#else
on_preferences_activate (GtkMenuItem * menuitem, gpointer user_data)
#endif
{
  GtkWidget *dialog;

  if (preferences_dialog_shown)
    return;
  preferences_dialog_shown = TRUE;
  dialog = create_preferences_dialog ();
  gtk_widget_show_all (dialog);
}


#ifdef USE_GNOME
void
preferences_dialog_clicked (GnomeDialog *dialog, gint button, gpointer user_data)
{
  switch (button){
    case 0: /* OK */
      change_preferences (GTK_WIDGET(dialog));
      if (preferences_changed)
        save_preferences ();
      preferences_dialog_shown = FALSE;
      if (dialog) gnome_dialog_close (dialog);
      break;
    case 1: /* APPLY */
      change_preferences (GTK_WIDGET(dialog));
      if (preferences_changed)
        save_preferences ();
      break;
    case 2: /* CLOSE */
      preferences_dialog_shown = FALSE;
      if (dialog) gnome_dialog_close (dialog);
      break;
  }
}

/*
void
on_preferences_apply_button_clicked (GnomePropertyBox *box, gint page_num, gpointer user_data)
{
  GtkWidget *preferences_dialog;

  preferences_dialog = gtk_widget_get_toplevel (GTK_WIDGET (box));
  change_preferences (preferences_dialog);
}
void
fontselection_changed (GtkFontSelection * fontselection, gpointer user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(gtk_widget_get_toplevel (GTK_WIDGET (fontselection))));
}
 */
#else
void
on_preferences_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *preferences_dialog;

  preferences_dialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
  change_preferences (preferences_dialog);
  if (preferences_changed)
    save_preferences ();
  preferences_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

void
on_preferences_apply_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *preferences_dialog;

  preferences_dialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
  change_preferences (preferences_dialog);
  if (preferences_changed)
    save_preferences ();
}

void
on_preferences_close_button_clicked (GtkButton * button, gpointer user_data)
{
  preferences_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}
#endif

void
preferences_dialog_destroy (GtkWidget *widget, gpointer user_data)
{
  preferences_dialog_shown = FALSE;
}

void
#ifdef USE_GNOME
on_preferences_button_clicked (GtkWidget * button, gpointer user_data)
#else
on_preferences_button_clicked (GtkButton * button, gpointer user_data)
#endif
{
  GtkWidget *dialog;

  if (preferences_dialog_shown)
    return;
  preferences_dialog_shown = TRUE;
  dialog = create_preferences_dialog ();
  gtk_widget_show_all (dialog);
}

void
on_main_window_draw (GtkWidget * widget,
		     GdkRectangle * area, gpointer user_data)
{
  redraw_all_cells ();
}


gboolean
on_drawingarea1_expose_event (GtkWidget * widget,
			      GdkEventExpose * event, gpointer user_data)
{
  redraw_all_cells ();
  return FALSE;
}

void
on_numkeys_radiobutton_clicked (GtkButton * button, gpointer user_data)
{
  use_graykeys = 0;
  preferences_changed = TRUE;
}


void
on_graykeys_radiobutton_clicked (GtkButton * button, gpointer user_data)
{
  use_graykeys = 1;
  preferences_changed = TRUE;
}

#ifdef USE_GNOME
void
on_sound_checkbutton_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  sound_on = !sound_on;
  preferences_changed = TRUE;
}
#endif
